#!/usr/bin/env python3

"""
Copyright (C) 2021 VMware, Inc.  All rights reserved. -- VMware Confidential
"""

import logging
import os
import pathlib
import requests
import sys
import time
import json

logging.basicConfig(format='%(asctime)s %(message)s', level=logging.DEBUG)

CONFIG_DIR = 'config'
SCRIPTS_DIR = 'scripts'


def enable_bundles(config_file_path, env):
    try:
        # Fetch required build info from deployment config
        with open(config_file_path) as handle:
            data = json.load(handle)
            logging.debug('deployment config content: {0}'.format(data))

        upgrade_criteria = data.get('upgrade_criteria', [])
        logging.debug('{0} upgrade criteria found for env {1}'.format(
            len(upgrade_criteria), env))

        count = 0
        for item in upgrade_criteria:
            org_key = item.get("org_key")
            appliance_uuid = item.get("appliance_uuid")
            tag = item.get("tag")
            version = item.get("version")

            # Validate upgrade criteria
            if not org_key or not appliance_uuid or not tag or not version:
                logging.error(
                    'One or more mandatory fields not specified in upgrade criteria: org_key, appliance_uuid, tag')
                sys.exit(1)

            payload = {
                "s3Key": 'cwp-appliance-upgrade-bundles/{0}/{1}/cwp-appliance-bundle-wrapper-{1}.zip'.format(env,
                                                                                                             version),
                "org_key": org_key,
                "applianceUUID": appliance_uuid,
                "tag": tag
            }

            ingress_api_url = os.environ['INGRESS_HOSTNAME']
            logging.debug('Backend API URL: {0}'.format(ingress_api_url))

            auth_token = '{0}/{1}'.format(
                os.environ['AUTH_TOKEN_API_KEY'], os.environ['AUTH_TOKEN_CONNECTOR_ID'])
            headers = {'X-Auth-Token': auth_token,
                       'Content-Type': 'application/json'}

            response = requests.put(
                'http://{0}/applianceservice/v1/upgradeCriteria'.format(ingress_api_url), data=json.dumps(payload),
                headers=headers)

            if response.status_code != 200:
                logging.warn('Failed to set upgrade criteria {0}, status code: {1}, message: {2}'.format(
                    payload, response.status_code, response.text))
            else:
                count = count + 1
                logging.info(
                    'Upgrade criteria {0} set successfully'.format(item))

        logging.info('{0} out of {1} upgrade criteria set succesfully'.format(
            count, len(upgrade_criteria)))

    except requests.exceptions.RequestException as e:
        logging.error('Failed to make HTTP request - {0}'.format(str(e)))
        sys.exit(1)
    except FileNotFoundError:
        logging.error('Config file not found {0}'.format(config_file_path))
    except json.decoder.JSONDecodeError:
        logging.error(
            'Invalid deployment config file {0}'.format(config_file_path))
        sys.exit(1)


def main():
    # Wait for instance metadata URL to be ready
    metadata_url = 'http://169.254.169.254/latest/meta-data/iam/security-credentials/'
    for _ in range(20):
        response = requests.get(metadata_url)
        if response.status_code != 200:
            logging.info('Waiting for security credentials to be available...')
            time.sleep(5)
        else:
            logging.info('Security credentials available')
            break
    else:
        logging.error(
            'Timed out waiting for security credentials to be available')
        sys.exit(1)

    script_path = pathlib.Path(__file__)
    logging.debug('script_path: {0}'.format(script_path))

    launch_vehicle_root = script_path.parent.parent.parent.absolute()
    logging.debug('launch_vehicle_root: {0}'.format(launch_vehicle_root))

    env_vars_to_check = ['INGRESS_HOSTNAME',
                         'AUTH_TOKEN_CONNECTOR_ID',
                         'AUTH_TOKEN_API_KEY',
                         'ENVIRONMENT']

    for env_var in env_vars_to_check:
        if env_var not in os.environ:
            logging.error('{0} is not defined'.format(env_var))
            sys.exit(1)

    environment = os.environ['ENVIRONMENT']
    logging.info('environment: {0}'.format(environment))

    config_file = 'deployment-{0}.json'.format(environment)
    logging.info('config_file: {0}'.format(config_file))

    deployment_root = script_path.parent.parent.absolute()
    logging.debug('deployment_root: {0}'.format(deployment_root))

    deployment_config_path = os.path.join(
        deployment_root, CONFIG_DIR, config_file)
    logging.debug('deployment_config_path: {0}'.format(deployment_config_path))

    enable_bundles(deployment_config_path, environment)


if __name__ == '__main__':
    main()
